8183554: Add constructors with Charset parameter for FileReader and FileWriter
authorjoehw
Thu, 15 Mar 2018 14:54:10 -0700
changeset 49254 422615764e12
parent 49253 26f624b33218
child 49255 acdb8531cc8b
8183554: Add constructors with Charset parameter for FileReader and FileWriter Reviewed-by: alanb, rriggs, smarks
src/java.base/share/classes/java/io/FileReader.java
src/java.base/share/classes/java/io/FileWriter.java
test/jdk/java/io/FileReader/ConstructorTest.java
test/jdk/java/io/FileWriter/ConstructorTest.java
--- /dev/null	Thu Jan 01 00:00:00 1970 +0000
+++ b/test/jdk/java/io/FileReader/ConstructorTest.java	Thu Mar 15 14:54:10 2018 -0700
@@ -0,0 +1,127 @@
+/*
+ * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
+ *
+ * This code is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 only, as
+ * published by the Free Software Foundation.
+ *
+ * This code is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * version 2 for more details (a copy is included in the LICENSE file that
+ * accompanied this code).
+ *
+ * You should have received a copy of the GNU General Public License version
+ * 2 along with this work; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
+ */
+
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileReader;
+import java.io.FileWriter;
+import java.io.IOException;
+import java.io.InputStreamReader;
+import java.io.Reader;
+import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
+import org.testng.Assert;
+import org.testng.annotations.DataProvider;
+import org.testng.annotations.Test;
+
+/**
+ * @test
+ * @bug 8183554
+ * @summary Test to verify the new Constructors that take a Charset.
+ * @run testng ConstructorTest
+ */
+public class ConstructorTest {
+    static String USER_DIR = System.getProperty("user.dir", ".");
+
+    public static enum ConstructorType {
+        STRING,
+        FILE
+    }
+
+    static final String TEST_STRING = "abc \u0100 \u0101 \u0555 \u07FD \u07FF";
+    static final int BUFFER_SIZE = 8192;
+
+    @DataProvider(name = "parameters")
+    public Object[][] getParameters() throws IOException {
+        File file1 = new File(USER_DIR, "FileReaderTest1.txt");
+        File file2 = new File(USER_DIR, "FileReaderTest2.txt");
+
+        return new Object[][]{
+            {ConstructorType.STRING, file1, file2, StandardCharsets.UTF_8},
+            {ConstructorType.FILE, file1, file2, StandardCharsets.UTF_8},
+            {ConstructorType.STRING, file1, file2, StandardCharsets.ISO_8859_1},
+            {ConstructorType.FILE, file1, file2, StandardCharsets.ISO_8859_1},
+        };
+    }
+
+    /**
+     * Verifies that the new constructors that take a Charset function the same
+     * as an InputStreamReader on a FileInputStream as was recommended before
+     * this change.
+     *
+     * @param type the type of the constructor
+     * @param file1 file1 to be read with a FileReader
+     * @param file2 file2 to be read with an InputStreamReader
+     * @param charset the charset
+     * @throws IOException
+     */
+    @Test(dataProvider = "parameters")
+    void test(ConstructorType type, File file1, File file2, Charset charset)
+            throws Exception {
+        prepareFile(file1, TEST_STRING, charset);
+        prepareFile(file2, TEST_STRING, charset);
+
+        try (FileReader fr = getFileReader(type, file1, charset);
+                FileInputStream is = new FileInputStream(file2);
+                InputStreamReader isr = new InputStreamReader(is, charset);) {
+            String result1 = readAll(fr, BUFFER_SIZE);
+            String result2 = readAll(isr, BUFFER_SIZE);
+            Assert.assertEquals(result1, result2);
+        }
+    }
+
+    public String readAll(Reader reader, int bufferSize) throws IOException {
+        StringBuilder sb = new StringBuilder();
+        char[] buf = new char[bufferSize];
+        int numRead;
+        while ((numRead = reader.read(buf)) != -1) {
+            if (numRead == buf.length) {
+                sb.append(buf);
+            } else {
+                sb.append(String.valueOf(buf, 0, numRead));
+            }
+        }
+        return sb.toString();
+    }
+
+    /*
+     * Creates a FileReader over the given input file.
+     */
+    FileReader getFileReader(ConstructorType type, File file, Charset charset)
+            throws IOException {
+        switch (type) {
+            case STRING:
+                return new FileReader(file.getPath(), charset);
+            case FILE:
+                return new FileReader(file, charset);
+        }
+
+        return null;
+    }
+
+    void prepareFile(File file, String content, Charset charset) throws IOException {
+        try (FileWriter writer = new FileWriter(file, charset);) {
+            writer.write(content);
+        }
+    }
+}
--- /dev/null	Thu Jan 01 00:00:00 1970 +0000
+++ b/test/jdk/java/io/FileWriter/ConstructorTest.java	Thu Mar 15 14:54:10 2018 -0700
@@ -0,0 +1,172 @@
+/*
+ * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
+ *
+ * This code is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 only, as
+ * published by the Free Software Foundation.
+ *
+ * This code is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * version 2 for more details (a copy is included in the LICENSE file that
+ * accompanied this code).
+ *
+ * You should have received a copy of the GNU General Public License version
+ * 2 along with this work; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
+ */
+
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.FileReader;
+import java.io.FileWriter;
+import java.io.IOException;
+import java.io.OutputStreamWriter;
+import java.io.Reader;
+import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
+import org.testng.Assert;
+import org.testng.annotations.DataProvider;
+import org.testng.annotations.Test;
+
+/**
+ * @test
+ * @bug 8183554
+ * @summary Test to verify the new Constructors that take a Charset.
+ * @run testng ConstructorTest
+ */
+public class ConstructorTest {
+    static String USER_DIR = System.getProperty("user.dir", ".");
+
+    public static enum ConstructorType {
+        STRING,
+        FILE,
+        STRING_APPEND,
+        FILE_APPEND
+    }
+
+    static final String TEST_STRING = "abc \u0100 \u0101 \u0555 \u07FD \u07FF";
+    static final int BUFFER_SIZE = 8192;
+
+    @DataProvider(name = "parameters")
+    public Object[][] getParameters() throws IOException {
+        File file1 = new File(USER_DIR, "FileWriterTest1.txt");
+        File file2 = new File(USER_DIR, "FileWriterTest2.txt");
+
+        return new Object[][]{
+            {ConstructorType.STRING, file1, file2, StandardCharsets.UTF_8},
+            {ConstructorType.FILE, file1, file2, StandardCharsets.UTF_8},
+            {ConstructorType.STRING_APPEND, file1, file2, StandardCharsets.UTF_8},
+            {ConstructorType.FILE_APPEND, file1, file2, StandardCharsets.UTF_8},
+            {ConstructorType.STRING, file1, file2, StandardCharsets.ISO_8859_1},
+            {ConstructorType.FILE, file1, file2, StandardCharsets.ISO_8859_1},
+            {ConstructorType.STRING_APPEND, file1, file2, StandardCharsets.ISO_8859_1},
+            {ConstructorType.FILE_APPEND, file1, file2, StandardCharsets.ISO_8859_1},
+        };
+    }
+
+    /**
+     * Verifies that the new constructors that take a Charset function the same
+     * as an OutputStreamWriter on a FileOutputStream as was recommended before
+     * this change.
+     *
+     * @param type the type of the constructor
+     * @param file1 file1 to be written with a FileWriter
+     * @param file2 file2 to be written  with an OutputStreamWriter
+     * @param charset the charset
+     * @throws IOException
+     */
+    @Test(dataProvider = "parameters")
+    void test(ConstructorType type, File file1, File file2, Charset charset)
+            throws Exception {
+        writeWithFileWriter(type, file1, TEST_STRING, charset);
+        writeWithOutputStreamWriter(type, file2, TEST_STRING, charset);
+
+        try (
+                FileReader r1 = getFileReader(type, file1, charset);
+                FileReader r2 = getFileReader(type, file2, charset);
+            ) {
+            String result1 = readAll(r1, BUFFER_SIZE);
+            String result2 = readAll(r2, BUFFER_SIZE);
+            Assert.assertEquals(result1, result2);
+        }
+    }
+
+    public String readAll(Reader reader, int bufferSize) throws IOException {
+        StringBuilder sb = new StringBuilder();
+        char[] buf = new char[bufferSize];
+        int numRead;
+        while ((numRead = reader.read(buf)) != -1) {
+            if (numRead == buf.length) {
+                sb.append(buf);
+            } else {
+                sb.append(String.valueOf(buf, 0, numRead));
+            }
+        }
+        return sb.toString();
+    }
+
+    /*
+     * Creates a FileReader over the given input file.
+     */
+    FileReader getFileReader(ConstructorType type, File file, Charset charset)
+            throws IOException {
+        switch (type) {
+            case STRING:
+            case STRING_APPEND:
+                return new FileReader(file.getPath(), charset);
+            case FILE:
+            case FILE_APPEND:
+                return new FileReader(file, charset);
+        }
+
+        return null;
+    }
+
+    /*
+     * Creates a FileWriter using the constructor as specified.
+     */
+    FileWriter getFileWriter(ConstructorType type, File file, Charset charset)
+            throws IOException {
+        switch (type) {
+            case STRING:
+                return new FileWriter(file.getPath(), charset);
+            case FILE:
+                return new FileWriter(file, charset);
+            case STRING_APPEND:
+                return new FileWriter(file.getPath(), charset, true);
+            case FILE_APPEND:
+                return new FileWriter(file, charset, true);
+        }
+
+        return null;
+    }
+
+    void writeWithFileWriter(ConstructorType type, File file, String content, Charset charset)
+            throws IOException {
+        if (type == ConstructorType.STRING_APPEND || type == ConstructorType.FILE_APPEND) {
+            try (FileWriter writer = getFileWriter(ConstructorType.FILE, file, charset);) {
+                writer.write(content);
+            }
+        }
+        try (FileWriter writer = getFileWriter(type, file, charset);) {
+            writer.write(content);
+        }
+    }
+
+    void writeWithOutputStreamWriter(ConstructorType type, File file, String content, Charset charset)
+            throws IOException {
+        try (OutputStreamWriter writer = new OutputStreamWriter(new FileOutputStream(file), charset)) {
+            writer.write(content);
+            if (type == ConstructorType.STRING_APPEND || type == ConstructorType.FILE_APPEND) {
+                writer.write(content);
+            }
+        }
+    }
+}
--- a/src/java.base/share/classes/java/io/FileReader.java	Thu Mar 15 19:33:04 2018 +0100
+++ b/src/java.base/share/classes/java/io/FileReader.java	Thu Mar 15 14:54:10 2018 -0700
@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2001, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,16 +25,17 @@
 
 package java.io;
 
+import java.nio.charset.Charset;
 
 /**
- * Convenience class for reading character files.  The constructors of this
- * class assume that the default character encoding and the default byte-buffer
- * size are appropriate.  To specify these values yourself, construct an
- * InputStreamReader on a FileInputStream.
+ * Reads text from character files using a default buffer size. Decoding from bytes
+ * to characters uses either a specified {@linkplain java.nio.charset.Charset charset}
+ * or the platform's
+ * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
  *
- * <p>{@code FileReader} is meant for reading streams of characters.
- * For reading streams of raw bytes, consider using a
- * {@code FileInputStream}.
+ * <p>
+ * The {@code FileReader} is meant for reading streams of characters. For reading
+ * streams of raw bytes, consider using a {@code FileInputStream}.
  *
  * @see InputStreamReader
  * @see FileInputStream
@@ -45,10 +46,11 @@
 public class FileReader extends InputStreamReader {
 
    /**
     * Creates a new {@code FileReader}, given the name of the
     * file to read from.
     *
-    * @param fileName the name of the file to read from
+    * @param fileName the name of the file to read
     * @exception  FileNotFoundException  if the named file does not exist,
     *                   is a directory rather than a regular file,
     *                   or for some other reason cannot be opened for
@@ -59,10 +61,11 @@
     }
 
    /**
-    * Creates a new {@code FileReader}, given the {@code File}
-    * to read from.
+    * Creates a new {@code FileReader}, given the {@code File} to read,
+    * using the platform's
+    * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
     *
-    * @param file the {@code File} to read from
+    * @param file the {@code File} to read
     * @exception  FileNotFoundException  if the file does not exist,
     *                   is a directory rather than a regular file,
     *                   or for some other reason cannot be opened for
@@ -73,13 +76,47 @@
     }
 
    /**
     * Creates a new {@code FileReader}, given the
     * {@code FileDescriptor} to read from.
     *
-    * @param fd the FileDescriptor to read from
+    * @param fd the {@code FileDescriptor} to read
     */
     public FileReader(FileDescriptor fd) {
         super(new FileInputStream(fd));
     }
 
+   /**
+    * Creates a new {@code FileReader}, given the name of the file to read
+    * and the {@linkplain java.nio.charset.Charset charset}.
+    *
+    * @param fileName the name of the file to read
+    * @param charset the {@linkplain java.nio.charset.Charset charset}
+    * @exception  IOException  if the named file does not exist,
+    *                   is a directory rather than a regular file,
+    *                   or for some other reason cannot be opened for
+    *                   reading.
+    *
+    * @since 11
+    */
+    public FileReader(String fileName, Charset charset) throws IOException {
+        super(new FileInputStream(fileName), charset);
+    }
+
+   /**
+    * Creates a new {@code FileReader}, given the {@code File} to read and
+    * the {@linkplain java.nio.charset.Charset charset}.
+    *
+    * @param file the {@code File} to read
+    * @param charset the {@linkplain java.nio.charset.Charset charset}
+    * @exception  IOException  if the file does not exist,
+    *                   is a directory rather than a regular file,
+    *                   or for some other reason cannot be opened for
+    *                   reading.
+    *
+    * @since 11
+    */
+    public FileReader(File file, Charset charset) throws IOException {
+        super(new FileInputStream(file), charset);
+    }
 }
@@ -45,10 +46,11 @@
 
    /**
-    * Creates a new {@code FileReader}, given the name of the
-    * file to read from.
+    * Creates a new {@code FileReader}, given the name of the file to read,
+    * using the platform's
+    * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
     *
@@ -73,13 +76,47 @@
 
    /**
-    * Creates a new {@code FileReader}, given the
-    * {@code FileDescriptor} to read from.
+    * Creates a new {@code FileReader}, given the {@code FileDescriptor} to read,
+    * using the platform's
+    * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
     *
--- a/src/java.base/share/classes/java/io/FileWriter.java	Thu Mar 15 19:33:04 2018 +0100
+++ b/src/java.base/share/classes/java/io/FileWriter.java	Thu Mar 15 14:54:10 2018 -0700
@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2001, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,22 +25,24 @@
 
 package java.io;
 
+import java.nio.charset.Charset;
 
 /**
- * Convenience class for writing character files.  The constructors of this
- * class assume that the default character encoding and the default byte-buffer
- * size are acceptable.  To specify these values yourself, construct an
- * OutputStreamWriter on a FileOutputStream.
+ * Writes text to character files using a default buffer size. Encoding from characters
+ * to bytes uses either a specified {@linkplain java.nio.charset.Charset charset}
+ * or the platform's
+ * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
  *
- * <p>Whether or not a file is available or may be created depends upon the
+ * <p>
+ * Whether or not a file is available or may be created depends upon the
  * underlying platform.  Some platforms, in particular, allow a file to be
  * opened for writing by only one {@code FileWriter} (or other file-writing
  * object) at a time.  In such situations the constructors in this class
  * will fail if the file involved is already open.
  *
- * <p>{@code FileWriter} is meant for writing streams of characters.
- * For writing streams of raw bytes, consider using a
- * {@code FileOutputStream}.
+ * <p>
+ * The {@code FileWriter} is meant for writing streams of characters. For writing
+ * streams of raw bytes, consider using a {@code FileOutputStream}.
  *
  * @see OutputStreamWriter
  * @see FileOutputStream
@@ -52,7 +54,8 @@
 public class FileWriter extends OutputStreamWriter {
 
     /**
-     * Constructs a FileWriter object given a file name.
+     * Constructs a {@code FileWriter} given a file name, using the platform's
+     * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}
      *
      * @param fileName  String The system-dependent filename.
      * @throws IOException  if the named file exists but is a directory rather
@@ -64,8 +67,9 @@
     }
 
     /**
      * Constructs a FileWriter object given a file name with a boolean
      * indicating whether or not to append the data written.
      *
      * @param fileName  String The system-dependent filename.
      * @param append    boolean if {@code true}, then data will be written
@@ -79,9 +83,11 @@
     }
 
     /**
-     * Constructs a FileWriter object given a File object.
+     * Constructs a {@code FileWriter} given the {@code File} to write,
+     * using the platform's
+     * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}
      *
-     * @param file  a File object to write to.
+     * @param file  the {@code File} to write.
      * @throws IOException  if the file exists but is a directory rather than
      *                  a regular file, does not exist but cannot be created,
      *                  or cannot be opened for any other reason
@@ -91,11 +97,11 @@
     }
 
     /**
-     * Constructs a FileWriter object given a File object. If the second
-     * argument is {@code true}, then bytes will be written to the end
-     * of the file rather than the beginning.
+     * Constructs a {@code FileWriter} given the {@code File} to write and
+     * a boolean indicating whether to append the data written, using the platform's
+     * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
      *
-     * @param file  a File object to write to
+     * @param file  the {@code File} to write
      * @param     append    if {@code true}, then bytes will be written
      *                      to the end of the file rather than the beginning
      * @throws IOException  if the file exists but is a directory rather than
@@ -108,12 +114,83 @@
     }
 
     /**
-     * Constructs a FileWriter object associated with a file descriptor.
+     * Constructs a {@code FileWriter} given a file descriptor,
+     * using the platform's
+     * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
      *
-     * @param fd  FileDescriptor object to write to.
+     * @param fd  the {@code FileDescriptor} to write.
      */
     public FileWriter(FileDescriptor fd) {
         super(new FileOutputStream(fd));
     }
 
+
+    /**
+     * Constructs a {@code FileWriter} given a file name and
+     * {@linkplain java.nio.charset.Charset charset}.
+     *
+     * @param fileName  the name of the file to write
+     * @param charset the {@linkplain java.nio.charset.Charset charset}
+     * @throws IOException  if the named file exists but is a directory rather
+     *                  than a regular file, does not exist but cannot be
+     *                  created, or cannot be opened for any other reason
+     *
+     * @since 11
+     */
+    public FileWriter(String fileName, Charset charset) throws IOException {
+        super(new FileOutputStream(fileName), charset);
+    }
+
+    /**
+     * Constructs a {@code FileWriter} given a file name,
+     * {@linkplain java.nio.charset.Charset charset} and a boolean indicating
+     * whether to append the data written.
+     *
+     * @param fileName  the name of the file to write
+     * @param charset the {@linkplain java.nio.charset.Charset charset}
+     * @param append    a boolean. If {@code true}, the writer will write the data
+     *                  to the end of the file rather than the beginning.
+     * @throws IOException  if the named file exists but is a directory rather
+     *                  than a regular file, does not exist but cannot be
+     *                  created, or cannot be opened for any other reason
+     *
+     * @since 11
+     */
+    public FileWriter(String fileName, Charset charset, boolean append) throws IOException {
+        super(new FileOutputStream(fileName, append), charset);
+    }
+
+    /**
+     * Constructs a {@code FileWriter} given the {@code File} to write and
+     * {@linkplain java.nio.charset.Charset charset}.
+     *
+     * @param file  the {@code File} to write
+     * @param charset the {@linkplain java.nio.charset.Charset charset}
+     * @throws IOException  if the file exists but is a directory rather than
+     *                  a regular file, does not exist but cannot be created,
+     *                  or cannot be opened for any other reason
+     *
+     * @since 11
+     */
+    public FileWriter(File file, Charset charset) throws IOException {
+        super(new FileOutputStream(file), charset);
+    }
+
+    /**
+     * Constructs a {@code FileWriter} given the {@code File} to write,
+     * {@linkplain java.nio.charset.Charset charset} and a boolean indicating
+     * whether to append the data written.
+     *
+     * @param file  the {@code File} to write
+     * @param charset the {@linkplain java.nio.charset.Charset charset}
+     * @param append    a boolean. If {@code true}, the writer will write the data
+     *                  to the end of the file rather than the beginning.
+     * @throws IOException  if the file exists but is a directory rather than
+     *                  a regular file, does not exist but cannot be created,
+     *                  or cannot be opened for any other reason
+     * @since 11
+     */
+    public FileWriter(File file, Charset charset, boolean append) throws IOException {
+        super(new FileOutputStream(file, append), charset);
+    }
 }
@@ -64,8 +67,9 @@
 
     /**
-     * Constructs a FileWriter object given a file name with a boolean
-     * indicating whether or not to append the data written.
+     * Constructs a {@code FileWriter} given a file name and a boolean indicating
+     * whether to append the data written, using the platform's
+     * {@linkplain java.nio.charset.Charset#defaultCharset() default charset}.
      *
      * @param fileName  String The system-dependent filename.